@@ -77,12 +77,34 @@
  * test is %TRUE. With the current set of available tests, there's no point
  * passing in more than one test at a time.
  * 
- * Apart from #G_FILE_TEST_IS_SYMLINK all tests follow symbolic links,
+ * Apart from %G_FILE_TEST_IS_SYMLINK all tests follow symbolic links,
  * so for a symbolic link to a regular file g_file_test() will return
- * %TRUE for both #G_FILE_TEST_IS_SYMLINK and #G_FILE_TEST_IS_REGULAR.
+ * %TRUE for both %G_FILE_TEST_IS_SYMLINK and %G_FILE_TEST_IS_REGULAR.
  *
  * Note, that for a dangling symbolic link g_file_test() will return
- * %TRUE for #G_FILE_TEST_IS_SYMLINK and %FALSE for all other flags.
+ * %TRUE for %G_FILE_TEST_IS_SYMLINK and %FALSE for all other flags.
+ *
+ * You should never use g_file_test() to test whether it is safe
+ * to perform an operaton, because there is always the possibility
+ * of the condition changing before you actually perform the operation.
+ * For example, you might think you could use %G_FILE_TEST_IS_SYMLINK
+ * to know whether it is is safe to write to a file without being
+ * tricked into writing into a different location. It doesn't work!
+ *
+ * <informalexample><programlisting>
+ * /&ast; DON'T DO THIS &ast;/
+ *  if (!g_file_test (filename, G_FILE_TEST_IS_SYMLINK)) {
+ *    fd = open (filename, O_WRONLY);
+ *    /&ast; write to fd &ast;/
+ *  }
+ * </programlisting></informalexample>
+ *
+ * Another thing to note is that %G_FILE_TEST_EXISTS and
+ * %G_FILE_TEST_IS_EXECUTABLE are implemented using the access()
+ * system call. This usually doesn't matter, but if your program
+ * is setuid or setgid it means that these tests will give you
+ * the answer for the real user ID and group ID , rather than the
+ * effective user ID and group ID.
  *
  * Return value: whether a test was %TRUE
  **/
@@ -94,7 +116,19 @@ g_file_test (const gchar *filename,
     return TRUE;
   
   if ((test & G_FILE_TEST_IS_EXECUTABLE) && (access (filename, X_OK) == 0))
-    return TRUE;
+    {
+#ifndef G_OS_WIN32
+      if (getuid () != 0)
+#endif	
+	return TRUE;
+
+      /* For root, on some POSIX systems, access (filename, X_OK)
+       * will succeed even if no executable bits are set on the
+       * file. We fall through to a stat test to avoid that.
+       */
+    }
+  else
+    test &= ~G_FILE_TEST_IS_EXECUTABLE;
 
   if (test & G_FILE_TEST_IS_SYMLINK)
     {
@@ -108,7 +142,9 @@ g_file_test (const gchar *filename,
 #endif
     }
   
-  if (test & (G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_DIR))
+  if (test & (G_FILE_TEST_IS_REGULAR |
+	      G_FILE_TEST_IS_DIR |
+	      G_FILE_TEST_IS_EXECUTABLE))
     {
       struct stat s;
       
@@ -119,6 +155,15 @@ g_file_test (const gchar *filename,
 	  
 	  if ((test & G_FILE_TEST_IS_DIR) && S_ISDIR (s.st_mode))
 	    return TRUE;
+
+	  /* The extra test for root when access (file, X_OK) succeeds.
+	   */
+	  if ((test & G_FILE_TEST_IS_EXECUTABLE) &&
+	      ((s.st_mode & S_IXOTH) ||
+	       (s.st_mode & S_IXUSR) ||
+	       (s.st_mode & S_IXGRP)))
+	    return TRUE;
+	      
 	}
     }
 
@@ -770,7 +815,10 @@ g_build_pathv (const gchar *separator,
   GString *result;
   gint separator_len = strlen (separator);
   gboolean is_first = TRUE;
+  gboolean have_leading = FALSE;
+  const gchar *single_element = NULL;
   const gchar *next_element;
+  const gchar *last_trailing = NULL;
 
   result = g_string_new (NULL);
 
@@ -790,49 +838,104 @@ g_build_pathv (const gchar *separator,
       else
 	break;
 
-      start = element;
+      /* Ignore empty elements */
+      if (!*element)
+	continue;
       
-      if (is_first)
-	is_first = FALSE;
-      else if (separator_len)
+      start = element;
+
+      if (separator_len)
 	{
 	  while (start &&
 		 strncmp (start, separator, separator_len) == 0)
 	    start += separator_len;
-	}
+      	}
 
       end = start + strlen (start);
       
-      if (next_element && separator_len)
+      if (separator_len)
 	{
-	  while (end > start + separator_len &&
+	  while (end >= start + separator_len &&
 		 strncmp (end - separator_len, separator, separator_len) == 0)
 	    end -= separator_len;
+	  
+	  last_trailing = end;
+	  while (last_trailing >= element + separator_len &&
+		 strncmp (last_trailing - separator_len, separator, separator_len) == 0)
+	    last_trailing -= separator_len;
+
+	  if (!have_leading)
+	    {
+	      /* If the leading and trailing separator strings are in the
+	       * same element and overlap, the result is exactly that element
+	       */
+	      if (last_trailing <= start)
+		single_element = element;
+		  
+	      g_string_append_len (result, element, start - element);
+	      have_leading = TRUE;
+	    }
+	  else
+	    single_element = NULL;
 	}
 
-      if (end > start)
-	{
-	  if (result->len > 0)
-	    g_string_append (result, separator);
+      if (end == start)
+	continue;
 
-	  g_string_append_len (result, start, end - start);
-	}
+      if (!is_first)
+	g_string_append (result, separator);
+      
+      g_string_append_len (result, start, end - start);
+      is_first = FALSE;
     }
+
+  if (single_element)
+    {
+      g_string_free (result, TRUE);
+      return g_strdup (single_element);
+    }
+  else
+    {
+      if (last_trailing)
+	g_string_append (result, last_trailing);
   
-  return g_string_free (result, FALSE);
+      return g_string_free (result, FALSE);
+    }
 }
 
 /**
  * g_build_path:
  * @separator: a string used to separator the elements of the path.
  * @first_element: the first element in the path
- * @Varargs: remaining elements in path
+ * @Varargs: remaining elements in path, terminated by %NULL
  * 
  * Creates a path from a series of elements using @separator as the
  * separator between elements. At the boundary between two elements,
  * any trailing occurrences of separator in the first element, or
  * leading occurrences of separator in the second element are removed
  * and exactly one copy of the separator is inserted.
+ *
+ * Empty elements are ignored.
+ *
+ * The number of leading copies of the separator on the result is
+ * the same as the number of leading copies of the separator on
+ * the first non-empty element.
+ *
+ * The number of trailing copies of the separator on the result is
+ * the same as the number of trailing copies of the separator on
+ * the last non-empty element. (Determination of the number of
+ * trailing copies is done without stripping leading copies, so
+ * if the separator is <literal>ABA</literal>, <literal>ABABA</literal>
+ * has 1 trailing copy.)
+ *
+ * However, if there is only a single non-empty element, and there
+ * are no characters in that element not part of the leading or
+ * trailing separators, then the result is exactly the original value
+ * of that element.
+ *
+ * Other than for determination of the number of leading and trailing
+ * copies of the separator, elements consisting only of copies
+ * of the separator are ignored.
  * 
  * Return value: a newly-allocated string that must be freed with g_free().
  **/
@@ -856,7 +959,7 @@ g_build_path (const gchar *separator,
 /**
  * g_build_filename:
  * @first_element: the first element in the path
- * @Varargs: remaining elements in path
+ * @Varargs: remaining elements in path, terminated by %NULL
  * 
  * Creates a filename from a series of elements using the correct
  * separator for filenames. This function behaves identically
