@@ -20,9 +20,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 
-#include "glib.h"
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -36,6 +35,8 @@
 #include <sys/select.h>
 #endif /* HAVE_SYS_SELECT_H */
 
+#include "glib.h"
+
 #include "glibintl.h"
 
 static gint g_execute (const gchar  *file,
